<?php
namespace Teknisa\CMP\Modules\Simplificada\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class CompraSimplificadaService 
{
	
	protected $entityManager;
	protected $environment;
	protected $dataSourceManager;
	protected $newCode;
	
	public function __construct(EntityManager $entityManager, $environment, DataSource\Manager $dataSourceManager, $newCode)
	{
		$this->entityManager = $entityManager;
		$this->environment = $environment;
		$this->dataSourceManager = $dataSourceManager;
		$this->newCode = $newCode;
	}

	public function iniciaTransacao()
	{
		$this->entityManager->beginTransaction();
	}

	public function commit()
	{
		$this->entityManager->commit();
	}

	public function flush()
	{
		$this->entityManager->flush();
	}

	public function rollback()
	{
		$this->entityManager->rollback();
	}
	
	public function getFiltroCompSimplificada()
	{
		$filterCriteria = new FilterCriteria(self::FILTROCOMPSIMPL);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet->getRows();
	}
	
	public function getNewCode($table)
	{
		return $this->newCode->getCodeByTableName($table, $this->environment->getNrOrgTrab());
	}
	
	public function getNrorg()
	{
		return $this->environment->getNrOrgTrab();
	}
	
	public function getOperadorLogado()
	{
		return $this->environment->getCdOperador();
	}
	
	public function getFilialLogada()
	{
		return $this->environment->getCdfilial();
	}
	
	private function getNextVal()
	{
		$connection = $this->entityManager->getConnection();
		$sql = "SELECT SEQ_FILTROCOMPSIMPL.NEXTVAL FROM DUAL";
		$query = $this->entityManager->getConnection()->prepare($sql);
		$query->execute();
		$result = $query->fetchAll();
		return is_array($result) && count($result) ? $result[0]['NEXTVAL'] : "";
	}
	
	public function getDataEntrega($nrfiltro)
	{
		$params = array(
    		"P_NRFILTROCOMPSFDA" => $nrfiltro,
    		"NRORG" => $this->environment->getNrorg()
    	);

    	$sql = "SELECT CDPERIENTR, NRORG, COUNT 
				  FROM (SELECT PE.CDPERIENTR, PE.NRORG, COUNT(1) OVER (PARTITION BY 1) COUNT
						  FROM PERFILPROD PE JOIN FILICOMPSIMPL FF ON PE.CDFILIAL = FF.CDFILIAL
											 JOIN PRODCOMPSIMPL FP ON FF.NRFILTROCOMPSFDA = FP.NRFILTROCOMPSFDA
						 WHERE FF.NRFILTROCOMPSFDA = :P_NRFILTROCOMPSFDA
						   AND PE.NRORG = :NRORG
						   AND FP.CDPRODUTO LIKE PE.CDPRODUTO||'%'
						 GROUP BY PE.CDPERIENTR, PE.NRORG)
				 WHERE ROWNUM = 1";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$periodicidade = $query->fetchAll();
    	
    	if (((is_array($periodicidade) && !is_null($periodicidade)) ? count($periodicidade) : 0) < 1) {
            return [];
		}
		
        return $periodicidade[0];
	}
	
	public function saveCompraSimplificadaFilter()
	{
		$sql = "INSERT INTO FILTROCOMPSIMPL (NRFILTROCOMPSFDA, CDPERIENTR, DIENTREGA,  DIACONSUMOINI,
                                                DIACONSUMOFIN, DTFILTRO, DTULTATUFILTRO, CDOPERRESP, NRORG)
                     VALUES (:NRFILTROCOMPSFDA, :CDPERIENTR, :DIENTREGA,  :DIACONSUMOINI,
                                :DIACONSUMOFIN, SYSDATE, SYSDATE, :CDOPERRESP, :NRORG)";
		
		try {
            $query = $this->entityManager->getConnection()->prepare($sql);
			$nrfiltro = $this->getNextVal();
			$query->bindValue("NRFILTROCOMPSFDA",   $nrfiltro);
			$query->bindValue("CDPERIENTR",         "01");
			$query->bindValue("DIENTREGA",          "01");
			$query->bindValue("DIACONSUMOINI",      "01");
			$query->bindValue("DIACONSUMOFIN",      "30");
			$query->bindValue("CDOPERRESP",         $this->environment->getCdOperador());
			$query->bindValue("NRORG",              $this->environment->getNrOrgTrab());
			$query->execute();
			
			return (float) $nrfiltro;
        } catch (Exception $e) {
            throw $e;
        }
	}
	
	public function deleteCompraSimplificadaFilter($nrfiltro)
	{
		$sql = "DELETE FROM FILTROCOMPSIMPL
				WHERE NRFILTROCOMPSFDA = :P_NRFILTROCOMPSFDA
				  AND NRORG = :NRORG";
				  
		try {
            $query = $this->entityManager->getConnection()->prepare($sql);
			$query->bindValue("P_NRFILTROCOMPSFDA", $nrfiltro);
			$query->bindValue("NRORG",              $this->environment->getNrOrgTrab());
			$query->execute();
        } catch (Exception $e) {
            throw $e;
        }
	}
	
	public function saveUnidadeSimplificadaFilter($nrfiltro, $cdfilial)
	{
		$sql = "INSERT INTO FILICOMPSIMPL 
		            (NRFILICOMPSIMPL, NRFILTROCOMPSFDA, CDFILIAL, NRORG) 
		        VALUES
		            (SEQ_FILICOMPSIMPL.NEXTVAL, :NRFILTROCOMPSFDA, :CDFILIAL, :NRORG)";
		
		try {
            $query = $this->entityManager->getConnection()->prepare($sql);
			$query->bindValue("NRFILTROCOMPSFDA",   $nrfiltro);
			$query->bindValue("CDFILIAL",           $cdfilial);
			$query->bindValue("NRORG",              $this->environment->getNrOrgTrab());
			$query->execute();
        } catch (Exception $e) {
            throw $e;
        }
	}
	
	public function saveProdutoSimplificadaFilter($nrfiltro, $produto, $nvproduto)
	{
		$sql = "INSERT INTO PRODCOMPSIMPL (NRPRODCOMPSIMPL, NRFILTROCOMPSFDA, CDPRODUTO, CDARVPROD, CDNVPRODUTO, NRORG) 
				SELECT SEQ_PRODCOMPSIMPL.NEXTVAL, :NRFILTROCOMPSFDA, PRODUTO.CDPRODUTO, :CDARVPROD, :CDNVPRODUTO, PRODUTO.NRORG
				  FROM PRODUTO
				 WHERE PRODUTO.CDNVPRODUTO = (SELECT MAX(PRODUTO.CDNVPRODUTO) FROM PRODUTO)
				   AND PRODUTO.CDPRODUTO LIKE :CDARVPROD || '%'
				   AND PRODUTO.NRORG = :NRORG";
		
		try {
            $query = $this->entityManager->getConnection()->prepare($sql);
			$query->bindValue("NRFILTROCOMPSFDA",   $nrfiltro);
			$query->bindValue("CDARVPROD",          $produto);
			$query->bindValue("CDNVPRODUTO",        $nvproduto);
			$query->bindValue("NRORG",              $this->environment->getNrOrgTrab());
			$query->execute();
        } catch (Exception $e) {
            throw $e;
        }
	}
	
	
	/*
	*
	* To do 
	* Implementar rotina de chamada da necessidade de compras
	*
	*
	*
	*/
	
	public function NECESSIDADE_DE_COMPRAS($datini, $datfin, $nececomp, $date, $nrcmpspl)
	{
		
		$operador = $this->environment->getCdoperador();
		$connection = $this->entityManager->getConnection();
		
		$procedure = new StoredProcedure($connection, 'NECESSIDADE_DE_COMPRAS');
		$procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, ' ',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_TCDFILIAL',       Param::PARAM_INPUT, ' ',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDGRUPFILI',      Param::PARAM_INPUT, ' ' ,       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTINICIAL',       Param::PARAM_INPUT, $datini,    Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTFINAL',         Param::PARAM_INPUT, $datfin,    Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDPRODINI',       Param::PARAM_INPUT, ' ',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDPRODFIN',       Param::PARAM_INPUT, ' ',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_PEACRESCIMO',     Param::PARAM_INPUT, 0,          Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $operador,  Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTHRCALCNC',      Param::PARAM_INPUT, $date,      Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_IDQTDEZERO',      Param::PARAM_INPUT, 'S',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRNECECOMP',      Param::PARAM_INPUT, $nececomp,  Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_EXTRA',           Param::PARAM_INPUT, 'N',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDSUBGRUPO',      Param::PARAM_INPUT, ' ',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDGRUPPROD',      Param::PARAM_INPUT, ' ',        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRCOMPSIMPLI',    Param::PARAM_INPUT, $nrcmpspl,  Param::PARAM_TYPE_STR));
		$procedure->execute();
	}
}
